@@ -9,6 +9,7 @@ import json
 import logging
 import time
 from datetime import datetime
+import traceback
 
 import asyncpg
 from asyncpg.exceptions import (
@@ -26,6 +27,7 @@ from asyncpg.exceptions import (
 )
 
 from asyncdb.exceptions import (
+    _handle_done_tasks,
     ConnectionTimeout,
     DataError,
     EmptyStatement,
@@ -39,9 +41,17 @@ from asyncdb.providers import (
     BaseProvider,
     registerProvider,
 )
+from asyncdb.utils.encoders import (
+    BaseEncoder,
+)
 from asyncdb.utils import (
-    EnumEncoder,
     SafeDict,
+    _escapeString,
+)
+
+from asyncdb.providers.sql import (
+    SQLProvider,
+    baseCursor
 )
 
 logger = logging.getLogger(__name__)
@@ -80,7 +90,7 @@ class pgPool(BasePool):
     async def init_connection(self, connection):
         # Setup jsonb encoder/decoder
         def _encoder(value):
-            return json.dumps(value, cls=EnumEncoder)
+            return json.dumps(value, cls=BaseEncoder)
 
         def _decoder(value):
             return json.loads(value)
@@ -112,7 +122,7 @@ class pgPool(BasePool):
             # TODO: pass a setup class for set_builtin_type_codec and a setup for add listener
             server_settings = {
                 "application_name": "Navigator",
-                "idle_in_transaction_session_timeout": "10000",
+                "idle_in_transaction_session_timeout": "600",
                 "tcp_keepalives_idle": "600",
                 "max_parallel_workers": "16",
             }
@@ -208,8 +218,8 @@ class pgPool(BasePool):
             conn = connection.engine()
         try:
             release = asyncio.create_task(self._pool.release(conn, timeout=10))
-            # await self._pool.release(conn, timeout = timeout)
-            # release = asyncio.ensure_future(release, loop=self._loop)
+            #await self._pool.release(conn, timeout = timeout)
+            #release = asyncio.ensure_future(release, loop=self._loop)
             await asyncio.wait_for(release, timeout=timeout, loop=self._loop)
         except InterfaceError as err:
             raise ProviderError("Release Interface Error: {}".format(str(err)))
@@ -239,19 +249,22 @@ class pgPool(BasePool):
                 )
             except Exception as err:
                 raise ProviderError("Release Error: {}".format(str(err)))
-            # at now, try to closing pool
             try:
                 if gracefully:
-                    await self._pool.expire_connections()
-                close = asyncio.create_task(self._pool.close())
-                await asyncio.wait_for(close, timeout=timeout, loop=self._loop)
+                    close = asyncio.create_task(self._pool.close())
+                    close.add_done_callback(_handle_done_tasks)
+                    await asyncio.wait_for(close, timeout=timeout, loop=self._loop)
+                else:
+                    await self._pool.close()
+            except asyncio.exceptions.TimeoutError as err:
+                print(traceback.format_exc())
             except Exception as err:
-                print("Pool Error: {}".format(str(err)))
-                await self._pool.terminate()
-                raise ProviderError("Pool Error: {}".format(str(err)))
+                error = f'Pool Exception: {err.__class__.__name__}: {err}'
+                print("Pool Error: {}".format(error))
+                raise ProviderError("Pool Error: {}".format(error))
             finally:
-                # await self._pool.terminate()
-                self._pool = None
+                self._pool.terminate()
+                self._connected = False
 
     """
     Close Pool
@@ -269,7 +282,9 @@ class pgPool(BasePool):
             await self._pool.close()
         except Exception as err:
             print("Pool Closing Error: {}".format(str(err)))
+        finally:
             self._pool.terminate()
+            self._connected = False
 
     def terminate(self, gracefully=True):
         self._loop.run_until_complete(
@@ -293,7 +308,18 @@ class pgPool(BasePool):
                 raise ProviderError("Execute Error: {}".format(str(err)))
 
 
-class pg(BaseProvider):
+class pglCursor(baseCursor):
+    _connection: asyncpg.Connection = None
+
+    async def __aenter__(self) -> "pglCursor":
+        if not self._connection:
+            await self.connection()
+        self._cursor = await self._connection.cursor(
+            self._sentence, self._params
+        )
+        return self
+
+class pg(SQLProvider):
     _provider = "postgresql"
     _syntax = "sql"
     _test_query = "SELECT 1"
@@ -303,28 +329,12 @@ class pg(BaseProvider):
     _transaction = None
     _initialized_on = None
     _query_raw = "SELECT {fields} FROM {table} {where_cond}"
-    """
-    Async Context magic Methods
-    """
-    async def __aenter__(self):
-        return self
+    _server_settings = {}
 
-    # async def __aexit__(self, exc_type, exc, tb):
-    async def __aexit__(self, *args):
-        # clean up anything you need to clean up
-        await self.close(timeout=5)
-        pass
-
-    """
-    Context magic Methods
-    """
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, type, value, traceback, *args):
-        self.terminate()
-        pass
+    def __init__(self, dsn: str = '', loop = None, params={}, **kwargs):
+        super(pg, self).__init__(dsn, loop, params, **kwargs)
+        if "server_settings" in kwargs:
+            self._server_settings = kwargs["server_settings"]
 
     async def close(self, timeout=5):
         """
@@ -366,16 +376,27 @@ class pg(BaseProvider):
         """
         Get a connection
         """
+        if self._connection:
+            return self
+
         self._connection = None
         self._connected = False
 
         # Setup jsonb encoder/decoder
         def _encoder(value):
-            return json.dumps(value, cls=EnumEncoder)
+            return json.dumps(value, cls=BaseEncoder)
 
         def _decoder(value):
             return json.loads(value)
 
+        server_settings = {
+            "application_name": "AsyncDB",
+            "idle_in_transaction_session_timeout": "600",
+            "tcp_keepalives_idle": "600",
+            "max_parallel_workers": "16",
+        }
+        server_settings = {**server_settings, **self._server_settings}
+
         try:
             if self._pool:
                 self._connection = await self._pool.pool().acquire()
@@ -386,6 +407,7 @@ class pg(BaseProvider):
                     timeout=self._timeout,
                     max_cached_statement_lifetime=max_cached_statement_lifetime,
                     max_cacheable_statement_size=max_cacheable_statement_size,
+                    server_settings=server_settings
                 )
                 await self._connection.set_type_codec(
                     "json",
@@ -635,6 +657,7 @@ class pg(BaseProvider):
     Cursor Context
     """
 
+
     async def cursor(self, sentence):
         if not sentence:
             raise EmptyStatement("Sentence is an empty string")
@@ -806,194 +829,6 @@ class pg(BaseProvider):
             error = "Error on Table Copy: {}".format(str(err))
             raise Exception(error)
 
-    """
-    Meta-Operations
-    """
-
-    def table(self, table):
-        try:
-            return self._query_raw.format_map(SafeDict(table=table))
-        except Exception as e:
-            print(e)
-            return False
-
-    def fields(self, sentence, fields=None):
-        _sql = False
-        if not fields:
-            _sql = sentence.format_map(SafeDict(fields="*"))
-        elif type(fields) == str:
-            _sql = sentence.format_map(SafeDict(fields=fields))
-        elif type(fields) == list:
-            _sql = sentence.format_map(SafeDict(fields=",".join(fields)))
-        return _sql
-
-    """
-    where
-      add WHERE conditions to SQL
-    """
-
-    def where(self, sentence, where):
-        sql = ""
-        if sentence:
-            where_string = ""
-            if not where:
-                sql = sentence.format_map(SafeDict(where_cond=""))
-            elif type(where) == dict:
-                where_cond = []
-                for key, value in where.items():
-                    # print("KEY {}, VAL: {}".format(key, value))
-                    if type(value) == str or type(value) == int:
-                        if value == "null" or value == "NULL":
-                            where_string.append("%s IS NULL" % (key))
-                        elif value == "!null" or value == "!NULL":
-                            where_string.append("%s IS NOT NULL" % (key))
-                        elif key.endswith("!"):
-                            where_cond.append("%s != %s" % (key[:-1], value))
-                        else:
-                            if (
-                                type(value) == str and value.startswith("'")
-                                and value.endswith("'")
-                            ):
-                                where_cond.append(
-                                    "%s = %s" % (key, "{}".format(value))
-                                )
-                            elif type(value) == int:
-                                where_cond.append(
-                                    "%s = %s" % (key, "{}".format(value))
-                                )
-                            else:
-                                where_cond.append(
-                                    "%s = %s" % (key, "'{}'".format(value))
-                                )
-                    elif type(value) == bool:
-                        val = str(value)
-                        where_cond.append("%s = %s" % (key, val))
-                    else:
-                        val = ",".join(map(str, value))
-                        if type(val) == str and "'" not in val:
-                            where_cond.append(
-                                "%s IN (%s)" % (key, "'{}'".format(val))
-                            )
-                        else:
-                            where_cond.append("%s IN (%s)" % (key, val))
-                # if 'WHERE ' in sentence:
-                #    where_string = ' AND %s' % (' AND '.join(where_cond))
-                # else:
-                where_string = " WHERE %s" % (" AND ".join(where_cond))
-                print("WHERE cond is %s" % where_string)
-                sql = sentence.format_map(SafeDict(where_cond=where_string))
-            elif type(where) == str:
-                where_string = where
-                if not where.startswith("WHERE"):
-                    where_string = " WHERE %s" % where
-                sql = sentence.format_map(SafeDict(where_cond=where_string))
-            else:
-                sql = sentence.format_map(SafeDict(where_cond=""))
-            del where
-            del where_string
-            return sql
-        else:
-            return False
-
-    def limit(self, sentence, limit=1):
-        """
-        LIMIT
-          add limiting to SQL
-        """
-        if sentence:
-            return "{q} LIMIT {limit}".format(q=sentence, limit=limit)
-        return self
-
-    def orderby(self, sentence, ordering=[]):
-        """
-        LIMIT
-          add limiting to SQL
-        """
-        if sentence:
-            if type(ordering) == str:
-                return "{q} ORDER BY {ordering}".format(
-                    q=sentence, ordering=ordering
-                )
-            elif type(ordering) == list:
-                return "{q} ORDER BY {ordering}".format(
-                    q=sentence, ordering=", ".join(ordering)
-                )
-        return self
-
-    def get_query(self, sentence):
-        """
-        get_query
-          Get formmated query
-        """
-        sql = sentence
-        try:
-            # remove fields and where_cond
-            sql = sentence.format_map(SafeDict(fields="*", where_cond=""))
-            if not self.connected:
-                self.connection()
-            prepared, error = self._loop.run_until_complete(self.prepare(sql))
-            if not error:
-                self._columns = self.get_columns()
-            else:
-                return False
-        except (ProviderError, StatementError) as err:
-            return False
-        except Exception as e:
-            print(e)
-            return False
-        return sql
-
-    def column_info(self, table):
-        """
-        column_info
-          get column information about a table
-        """
-        discover = "SELECT attname AS column_name, atttypid::regtype AS data_type FROM pg_attribute WHERE attrelid = '{}'::regclass AND attnum > 0 AND NOT attisdropped ORDER  BY attnum".format(
-            table
-        )
-        try:
-            result, error = self._loop.run_until_complete(self.query(discover))
-            if result:
-                return result
-        except (NoDataFound, ProviderError):
-            print(err)
-            return False
-        except Exception as err:
-            print(err)
-            return False
-
-    def insert(self, table, data, **kwargs):
-        """
-        insert
-           insert the result onto a table
-        """
-        sql = "INSERT INTO {table} ({fields}) VALUES ({values})"
-        sql = sql.format_map(SafeDict(table=table))
-        # set columns
-        sql = sql.format_map(SafeDict(fields=",".join(data.keys())))
-        values = ",".join(_escapeString(v) for v in data.values())
-        sql = sql.format_map(SafeDict(values=values))
-        # print(sql)
-        try:
-            result = self._loop.run_until_complete(
-                self._connection.execute(sql)
-            )
-            if not result:
-                print(result)
-                return False
-            else:
-                return result
-        except Exception as err:
-            print(sql)
-            print(err)
-            return False
-
-    def _escapeString(value):
-        v = value if value != "None" else ""
-        v = str(v).replace("'", "''")
-        v = "'{}'".format(v) if type(v) == str else v
-        return v
-
 
 """
 Registering this Provider
