@@ -21,6 +21,7 @@ from datetime import datetime
 from threading import Thread
 
 from rethinkdb import RethinkDB
+
 from rethinkdb.errors import (
     ReqlDriverError,
     ReqlError,
@@ -28,9 +29,7 @@ from rethinkdb.errors import (
     ReqlOpFailedError,
     ReqlOpIndeterminateError,
     ReqlResourceLimitError,
-    ReqlRuntimeError,
-    RqlDriverError,
-    RqlRuntimeError,
+    ReqlRuntimeError
 )
 
 from asyncdb.exceptions import (
@@ -42,15 +41,15 @@ from asyncdb.exceptions import (
     StatementError,
     TooManyConnections,
 )
-from asyncdb.utils import *
+from asyncdb.utils.functions import *
 
-from . import *
+from . import (
+    BaseProvider,
+    registerProvider,
+)
 
 rt = RethinkDB()
 
-logger = logging.getLogger(__name__)
-
-
 class rethink(BaseProvider):
     _provider = "rethinkdb"
     _syntax = "rql"
@@ -77,7 +76,10 @@ class rethink(BaseProvider):
 
     def __init__(self, loop=None, params={}, **kwargs):
         super(rethink, self).__init__(loop=loop, params=params)
-        self._loop = asyncio.get_event_loop()
+        if loop:
+            self._loop = loop
+        else:
+            self._loop = asyncio.get_event_loop()
         self.conditions = {}
         # set rt object
         self._engine = rt
@@ -85,15 +87,9 @@ class rethink(BaseProvider):
         self._engine.set_loop_type("asyncio")
         asyncio.set_event_loop(self._loop)
 
-    """
-    Async Context magic Methods
-    """
-
-    async def __aexit__(self, exc_type, exc, tb):
-        await self.close(wait=True)
 
     async def connection(self):
-        logger.debug(
+        logging.debug(
             "RT Connection to host {} on port {} to database {}".format(
                 self._params["host"], self._params["port"], self._params["db"]
             )
@@ -114,6 +110,12 @@ class rethink(BaseProvider):
             )
             raise ProviderError(message=error, code=503)
             return False
+        except Exception as err:
+            error = "Exception on RethinkDB: {}".format(
+                str(err)
+            )
+            raise ProviderError(message=error, code=503)
+            return False
         finally:
             if self._connection:
                 self._connected = True
@@ -137,9 +139,12 @@ class rethink(BaseProvider):
             self._connection = None
 
     def terminate(self):
-        logger.debug("Running Close")
+        logging.debug("Closing Rethink Connection")
         self._loop.run_until_complete(self.close())
 
+    async def release(self):
+        await self.close(wait=10)
+
     """
     Basic Methods
     """
@@ -222,7 +227,7 @@ class rethink(BaseProvider):
                         )
                     except ReqlOpFailedError as err:
                         raise ProviderError(message=err, code=503)
-                    # except (RqlDriverError, RqlRuntimeError):
+                    # except (ReqlDriverError, ReqlRuntimeError):
                     #    return False
                 elif type(fields) == list and len(fields) > 0:
                     idx = []
@@ -234,7 +239,7 @@ class rethink(BaseProvider):
                                 name, idx
                             ).run(self._connection)
                         )
-                    except (RqlDriverError, RqlRuntimeError):
+                    except (ReqlDriverError, ReqlRuntimeError):
                         return False
             else:
                 return False
@@ -261,15 +266,14 @@ class rethink(BaseProvider):
         finally:
             return self
 
-    async def clean(self, table):
+    async def clean(self, table, conditions: dict = {}):
         """
         clean
            Clean a Table
         """
-        conditions = {}
         result = []
         if self.conditions:
-            conditions = {**self.conditions}
+            conditions = {**conditions, **self.conditions}
 
         conditions.update(
             (x, None) for (x, y) in conditions.items() if y == "null"
@@ -322,15 +326,15 @@ class rethink(BaseProvider):
                 self._engine.db(self._db
                                 ).table_drop(table).run(self._connection)
             )
-        except (RqlDriverError, RqlRuntimeError) as err:
+        except (ReqlDriverError, ReqlRuntimeError) as err:
             raise ProviderError(str(err))
 
     """
     Derived Methods (mandatory)
     """
 
-    def test_connection(self):
-        pass
+    async def test_connection(self):
+        return await self._engine.db_list().run(self._connection)
 
     def execute(self):
         pass
@@ -381,7 +385,7 @@ class rethink(BaseProvider):
             except ReqlNonExistenceError as err:
                 error = "Query Runtime Error: {}".format(str(err))
                 raise ReqlNonExistenceError(error)
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "Query Runtime Error: {}".format(str(err))
                 raise NoDataFound(error)
             except ReqlResourceLimitError as error:
@@ -417,7 +421,7 @@ class rethink(BaseProvider):
             except ReqlNonExistenceError as err:
                 error = "Empty Result: {}".format(str(err))
                 raise NoDataFound(error)
-            except (RqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
+            except (ReqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
                 error = "QueryRow Runtime Error: {}".format(str(err))
                 raise ProviderError(err)
                 return False
@@ -455,7 +459,7 @@ class rethink(BaseProvider):
             except ReqlNonExistenceError as err:
                 error = "Empty Result: {}".format(str(err))
                 raise NoDataFound(error)
-            except (RqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
+            except (ReqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
                 error = "QueryRow Runtime Error: {}".format(str(err))
                 raise ProviderError(err)
                 return False
@@ -496,7 +500,7 @@ class rethink(BaseProvider):
                 else:
                     raise NoDataFound(message="Empty Result", code=404)
                 return self._result
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "Query Get All Runtime Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -522,7 +526,7 @@ class rethink(BaseProvider):
                 else:
                     raise NoDataFound(message="Empty Result", code=404)
                 return self._result
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "Query Get All Runtime Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -550,7 +554,7 @@ class rethink(BaseProvider):
                     )
                     return False
                 return inserted
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "INSERT Runtime Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -583,7 +587,7 @@ class rethink(BaseProvider):
                     )
                     return False
                 return self._result
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "REPLACE Runtime Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -610,7 +614,7 @@ class rethink(BaseProvider):
                     )
                     # self._result = await self._engine.table(table).get(id).update(data).run(self._connection)
                     return self._result
-                except RqlRuntimeError as err:
+                except ReqlRuntimeError as err:
                     error = "UPDATE Runtime Error: {}".format(str(err))
                     raise ProviderError(error)
                     return False
@@ -628,7 +632,7 @@ class rethink(BaseProvider):
                     )
                     # self._result = await self._engine.table(table).filter(filter).update(data).run(self._connection)
                     return self._result
-                except RqlRuntimeError as err:
+                except ReqlRuntimeError as err:
                     error = "REPLACE Runtime Error: {}".format(str(err))
                     raise ProviderError(error)
                     return False
@@ -660,7 +664,7 @@ class rethink(BaseProvider):
                     )
                 )
                 return self._result
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "Literal Runtime Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -686,7 +690,7 @@ class rethink(BaseProvider):
                     ).filter(filter).update(data).run(self._connection)
                 )
                 return self._result
-            except RqlRuntimeError as err:
+            except ReqlRuntimeError as err:
                 error = "Update Conditions Error: {}".format(str(err))
                 raise ProviderError(error)
                 return False
@@ -714,10 +718,10 @@ class rethink(BaseProvider):
                         ).run(self._connection)
                     )
                     return self._result
-                except (RqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
+                except (ReqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
                     raise ProviderError(err)
                     return False
-            elif type(filter) == dict and len(filter) > 0:
+            elif isinstance(filter, dict):
                 try:
                     self._result = (
                         await self._engine.table(table).filter(filter).delete(
@@ -725,7 +729,7 @@ class rethink(BaseProvider):
                         ).run(self._connection)
                     )
                     return self._result
-                except (RqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
+                except (ReqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
                     raise ProviderError(err)
                     return False
             else:
@@ -778,7 +782,7 @@ class rethink(BaseProvider):
                     self._result = data
                 else:
                     raise NoDataFound(message="Empty Result", code=404)
-            except (RqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
+            except (ReqlRuntimeError, ReqlRuntimeError, ReqlError) as err:
                 error = str(err)
                 raise ProviderError(err)
                 return False
@@ -1167,7 +1171,7 @@ class rethink(BaseProvider):
             try:
                 try:
                     cursor = await search.run(self._connection)
-                except (RqlRuntimeError, ReqlRuntimeError) as err:
+                except (ReqlRuntimeError, ReqlRuntimeError) as err:
                     print("Error on rql query is %s" % err.message)
                     raise Exception("Error on RQL query is %s" % err.message)
                     return False
